@@ -27,6 +27,7 @@
 #define BABL_INIT_HOOK    init_hook (); dynamic_init_hook ();
 #endif
 
+#define NEEDS_BABL_DB
 #include "babl-internal.h"
 #include "babl-db.h"
 #include "babl-base.h"
@@ -326,12 +327,15 @@ babl_extension_load_dir_list (const char *dir_list)
         {
           case '\0':
             eos = 1;
+            /* don't break here, the path needs to be processed */
 
           case BABL_PATH_SEPARATOR:
           {
             char *expanded_path = expand_path (path);
-            babl_extension_load_dir (expanded_path);
-            babl_free (expanded_path);
+            if (expanded_path) {
+                babl_extension_load_dir (expanded_path);
+                babl_free (expanded_path);
+            }
           }
             dst = path;
             src++;
