@@ -19,25 +19,35 @@
 #include "config.h"
 #include <math.h>
 #include "babl-internal.h"
+#include "babl-ref-pixels.h"
 
 #define BABL_LEGAL_ERROR           0.000001
 #define BABL_MAX_COST_VALUE        2000000
 #define BABL_HARD_MAX_PATH_LENGTH  8
 #define BABL_MAX_NAME_LEN          1024
 
+#ifndef MIN
+#define MIN(a, b) (((a) > (b)) ? (b) : (a))
+#endif
+
+#define NUM_TEST_PIXELS            (babl_get_num_path_test_pixels ())
+#define MAX_BUFFER_SIZE            1024  /* XXX: reasonable size for this should be profiled */
+
+
+int   babl_in_fish_path = 0;
 
 typedef struct _FishPathInstrumentation
 {
-  Babl   *fmt_rgba_double;
-  double *test;
+  const Babl   *fmt_rgba_double;
+  int     num_test_pixels;
   void   *source;
   void   *destination;
   void   *ref_destination;
   double *destination_rgba_double;
   double *ref_destination_rgba_double;
-  Babl   *fish_rgba_to_source;
-  Babl   *fish_reference;
-  Babl   *fish_destination_to_rgba;
+  const Babl   *fish_rgba_to_source;
+  const Babl   *fish_reference;
+  const Babl   *fish_destination_to_rgba;
   double  reference_cost;
   int     init_instrumentation_done;
 } FishPathInstrumentation;
@@ -63,11 +73,14 @@ get_path_instrumentation (FishPathInstrumentation *fpi,
                           double                  *ref_cost,
                           double                  *path_error);
 
+
 static long
-process_conversion_path (BablList *path,
-                         void     *source_buffer,
-                         void     *destination_buffer,
-                         long     n);
+process_conversion_path (BablList   *path,
+                         const void *source_buffer,
+                         int         source_bpp,
+                         void       *destination_buffer,
+                         int         dest_bpp,
+                         long        n);
 
 static void
 get_conversion_path (PathContext *pc,
@@ -75,9 +88,6 @@ get_conversion_path (PathContext *pc,
                      int current_length,
                      int max_length);
 
-static double *
-test_create (void);
-
 static char *
 create_name (char       *buf,
              const Babl *source,
@@ -99,7 +109,7 @@ static double legal_error (void)
 
   env = getenv ("BABL_TOLERANCE");
   if (env && env[0] != '\0')
-    error = atof (env);
+    error = babl_parse_double (env);
   else
     error = BABL_LEGAL_ERROR;
   return error;
@@ -172,6 +182,10 @@ get_conversion_path (PathContext *pc,
         {
           FishPathInstrumentation fpi;
           memset (&fpi, 0, sizeof (fpi));
+
+          fpi.source = (Babl*) babl_list_get_first (pc->current_path)->conversion.source;
+          fpi.destination = pc->to_format;
+
           get_path_instrumentation (&fpi, pc->current_path, &path_cost, &ref_cost, &path_error);
 
           if ((path_cost < ref_cost) && /* do not use paths that took longer to compute than reference */
@@ -286,15 +300,20 @@ babl_fish_path (const Babl *source,
     pc.fish_path = babl;
     pc.to_format = (Babl *) destination;
 
-    babl_mutex_lock (babl_format_mutex);
+    if (babl_in_fish_path <= 0)
+      babl_mutex_lock (babl_format_mutex);
     /* we hold a global lock whilerunning get_conversion_path since
      * it depends on keeping the various format.visited members in
      * a consistent state, this code path is not performance critical
      * since created fishes are cached.
      */
+    babl_in_fish_path++;
 
     get_conversion_path (&pc, (Babl *) source, 0, max_path_length ());
-    babl_mutex_unlock (babl_format_mutex);
+
+    babl_in_fish_path--;
+    if (babl_in_fish_path <= 0)
+      babl_mutex_unlock (babl_format_mutex);
     babl_free (pc.current_path);
   }
 
@@ -312,24 +331,54 @@ babl_fish_path (const Babl *source,
 }
 
 static long
-babl_fish_path_process (Babl *babl,
-                        void *source,
-                        void *destination,
-                        long  n)
+babl_fish_path_process (Babl       *babl,
+                        const void *source,
+                        void       *destination,
+                        long        n)
 {
+   const Babl *babl_source = babl->fish.source;
+   const Babl *babl_dest = babl->fish.destination;
+   int source_bpp = 0;
+   int dest_bpp = 0;
+
+   switch (babl_source->instance.class_type)
+     {
+       case BABL_FORMAT:
+         source_bpp = babl_source->format.bytes_per_pixel;
+         break;
+       case BABL_TYPE:
+         source_bpp = babl_source->type.bits / 8;
+         break;
+       default:
+         babl_log ("=eeek{%i}\n", babl_source->instance.class_type - BABL_MAGIC);
+     }
+
+   switch (babl_dest->instance.class_type)
+     {
+       case BABL_FORMAT:
+         dest_bpp = babl_dest->format.bytes_per_pixel;
+         break;
+       case BABL_TYPE:
+         dest_bpp = babl_dest->type.bits / 8;
+         break;
+       default:
+         babl_log ("-eeek{%i}\n", babl_dest->instance.class_type - BABL_MAGIC);
+     }
+
   return process_conversion_path (babl->fish_path.conversion_list,
-                                 source,
-                                 destination,
-                                 n);
+                                  source,
+                                  source_bpp,
+                                  destination,
+                                  dest_bpp,
+                                  n);
 
 }
 
-
 static long
-babl_fish_process (Babl *babl,
-                   void *source,
-                   void *destination,
-                   long  n)
+babl_fish_process (Babl       *babl,
+                   const void *source,
+                   void       *destination,
+                   long        n)
 {
   long ret = 0;
 
@@ -373,11 +422,12 @@ babl_fish_process (Babl *babl,
 }
 
 long
-babl_process (Babl *babl,
-              void *source,
-              void *destination,
-              long  n)
+babl_process (const Babl *cbabl,
+              const void *source,
+              void       *destination,
+              long        n)
 {
+  Babl *babl = (Babl*)cbabl;
   babl_assert (babl);
   babl_assert (source);
   babl_assert (destination);
@@ -386,13 +436,14 @@ babl_process (Babl *babl,
     return 0;
   babl_assert (n > 0);
 
-  /* first check if it is a fish since that is out fast path */
+  /* first check if it is a fish since that is our fast path */
   if (babl->class_type >= BABL_FISH &&
       babl->class_type <= BABL_FISH_PATH)
     {
       babl->fish.processings++;
-      return babl->fish.pixels +=
+      babl->fish.pixels +=
              babl_fish_process (babl, source, destination, n);
+      return n;
     }
 
   /* matches all conversion classes */
@@ -404,11 +455,23 @@ babl_process (Babl *babl,
   return -1;
 }
 
+#include <stdint.h>
+
+#define BABL_ALIGN 16
+static void inline *align_16 (unsigned char *ret)
+{
+  int offset = BABL_ALIGN - ((uintptr_t) ret) % BABL_ALIGN;
+  ret = ret + offset;
+  return ret;
+}
+
 static long
-process_conversion_path (BablList *path,
-                         void     *source_buffer,
-                         void     *destination_buffer,
-                         long     n)
+process_conversion_path (BablList   *path,
+                         const void *source_buffer,
+                         int         source_bpp,
+                         void       *destination_buffer,
+                         int         dest_bpp,
+                         long        n)
 {
   int conversions = babl_list_size (path);
 
@@ -421,86 +484,62 @@ process_conversion_path (BablList *path,
     }
   else
     {
-      void *aux1_buffer = babl_malloc (n * sizeof (double) * 5);
-      void *aux2_buffer = NULL;
-      void *swap_buffer = NULL;
-      int   i;
+      long j;
+
+      void *temp_buffer = align_16 (alloca (MIN(n, MAX_BUFFER_SIZE) * sizeof (double) * 5 + 16));
+      void *temp_buffer2 = NULL;
 
       if (conversions > 2)
         {
           /* We'll need one more auxiliary buffer */
-          aux2_buffer = babl_malloc (n * sizeof (double) * 5);
+          temp_buffer2 = align_16 (alloca (MIN(n, MAX_BUFFER_SIZE) * sizeof (double) * 5 + 16));
         }
 
-      /* The first conversion goes from source_buffer to aux1_buffer */
-      babl_conversion_process (babl_list_get_first (path),
-                               source_buffer,
-                               aux1_buffer,
-                               n);
 
-      /* Process, if any, conversions between the first and the last
-       * conversion in the path, in a loop */
-      for (i = 1; i < conversions - 1; i++)
+
+
+      for (j = 0; j < n; j+= MAX_BUFFER_SIZE)
         {
-          babl_conversion_process (path->items[i],
+          long c = MIN (n - j, MAX_BUFFER_SIZE);
+          int i;
+
+          /* this is where the loop unrolling should happen */
+          void *aux1_buffer = temp_buffer;
+          void *aux2_buffer = NULL;
+          void *swap_buffer = NULL;
+          aux2_buffer = temp_buffer2;
+
+          /* The first conversion goes from source_buffer to aux1_buffer */
+          babl_conversion_process (babl_list_get_first (path),
+                                   (void*)(((unsigned char*)source_buffer) + (j * source_bpp)),
                                    aux1_buffer,
-                                   aux2_buffer,
-                                   n);
-          /* Swap the auxiliary buffers */
-          swap_buffer = aux1_buffer;
-          aux1_buffer = aux2_buffer;
-          aux2_buffer = swap_buffer;
+                                   c);
+
+          /* Process, if any, conversions between the first and the last
+           * conversion in the path, in a loop */
+          for (i = 1; i < conversions - 1; i++)
+            {
+              babl_conversion_process (path->items[i],
+                                       aux1_buffer,
+                                       aux2_buffer,
+                                       c);
+              /* Swap the auxiliary buffers */
+              swap_buffer = aux1_buffer;
+              aux1_buffer = aux2_buffer;
+              aux2_buffer = swap_buffer;
+            }
+
+          /* The last conversion goes from aux1_buffer to destination_buffer */
+          babl_conversion_process (babl_list_get_last (path),
+                                   aux1_buffer,
+                                   (void*)((unsigned char*)destination_buffer + (j * dest_bpp)),
+                                   c);
         }
-
-      /* The last conversion goes from aux1_buffer to destination_buffer */
-      babl_conversion_process (babl_list_get_last (path),
-                               aux1_buffer,
-                               destination_buffer,
-                               n);
-
-      /* Free auxiliary buffers */
-      if (aux1_buffer)
-        babl_free (aux1_buffer);
-      if (aux2_buffer)
-        babl_free (aux2_buffer);
   }
 
   return n;
 }
 
-#define NUM_TEST_PIXELS  (256 + 16 + 16)
-
-static double *
-test_create (void)
-{
-  static double test[sizeof (double) * NUM_TEST_PIXELS * 4];
-  int           i, j;
-  static int    done = 0;
-
-  if (done)
-    return test;
-
-  /* There is no need to generate the test
-   * more times ... */
-
-  srandom (20050728);
-
-  /*  add 128 pixels in the valid range between 0.0 and 1.0  */
-  for (i = 0; i < 256 * 4; i++)
-    test [i] = (double) random () / RAND_MAX;
-
-  /*  add 16 pixels between -1.0 and 0.0  */
-  for (j = 0; j < 16 * 4; i++, j++)
-    test [i] = 0.0 - (double) random () / RAND_MAX;
-
-  /*  add 16 pixels between 1.0 and 2.0  */
-  for (j = 0; j < 16 * 4; i++, j++)
-    test [i] = 1.0 + (double) random () / RAND_MAX;
-
-  done = 1;
-  return test;
-}
-
 static void
 init_path_instrumentation (FishPathInstrumentation *fpi,
                            Babl                    *fmt_source,
@@ -509,6 +548,8 @@ init_path_instrumentation (FishPathInstrumentation *fpi,
   long   ticks_start = 0;
   long   ticks_end   = 0;
 
+  const double *test_pixels = babl_get_path_test_pixels ();
+
   if (!fpi->fmt_rgba_double)
     {
       fpi->fmt_rgba_double = babl_format_new (
@@ -521,8 +562,7 @@ init_path_instrumentation (FishPathInstrumentation *fpi,
         NULL);
     }
 
-  if (!fpi->test)
-    fpi->test = test_create (); // <- test_create utiliza var static dentro de la función
+  fpi->num_test_pixels = babl_get_num_path_test_pixels ();
 
   fpi->fish_rgba_to_source      = babl_fish_reference (fpi->fmt_rgba_double,
                                                   fmt_source);
@@ -531,31 +571,31 @@ init_path_instrumentation (FishPathInstrumentation *fpi,
   fpi->fish_destination_to_rgba = babl_fish_reference (fmt_destination,
                                                   fpi->fmt_rgba_double);
 
-  fpi->source                      = babl_calloc (NUM_TEST_PIXELS,
+  fpi->source                      = babl_calloc (fpi->num_test_pixels,
                                              fmt_source->format.bytes_per_pixel);
-  fpi->destination                 = babl_calloc (NUM_TEST_PIXELS,
+  fpi->destination                 = babl_calloc (fpi->num_test_pixels,
                                              fmt_destination->format.bytes_per_pixel);
-  fpi->ref_destination             = babl_calloc (NUM_TEST_PIXELS,
+  fpi->ref_destination             = babl_calloc (fpi->num_test_pixels,
                                              fmt_destination->format.bytes_per_pixel);
-  fpi->destination_rgba_double     = babl_calloc (NUM_TEST_PIXELS,
+  fpi->destination_rgba_double     = babl_calloc (fpi->num_test_pixels,
                                              fpi->fmt_rgba_double->format.bytes_per_pixel);
-  fpi->ref_destination_rgba_double = babl_calloc (NUM_TEST_PIXELS,
+  fpi->ref_destination_rgba_double = babl_calloc (fpi->num_test_pixels,
                                              fpi->fmt_rgba_double->format.bytes_per_pixel);
 
   /* create sourcebuffer from testbuffer in the correct format */
   babl_process (fpi->fish_rgba_to_source,
-                fpi->test, fpi->source, NUM_TEST_PIXELS);
+                test_pixels, fpi->source, fpi->num_test_pixels);
 
   /* calculate the reference buffer of how it should be */
   ticks_start = babl_ticks ();
   babl_process (fpi->fish_reference,
-                fpi->source, fpi->ref_destination, NUM_TEST_PIXELS);
+                fpi->source, fpi->ref_destination, fpi->num_test_pixels);
   ticks_end = babl_ticks ();
   fpi->reference_cost = babl_process_cost (ticks_start, ticks_end);
 
   /* transform the reference destination buffer to RGBA */
   babl_process (fpi->fish_destination_to_rgba,
-                fpi->ref_destination, fpi->ref_destination_rgba_double, NUM_TEST_PIXELS);
+                fpi->ref_destination, fpi->ref_destination_rgba_double, fpi->num_test_pixels);
 }
 
 static void
@@ -584,20 +624,48 @@ get_path_instrumentation (FishPathInstrumentation *fpi,
   long   ticks_start = 0;
   long   ticks_end   = 0;
 
+  Babl *babl_source = fpi->source;
+  Babl *babl_destination = fpi->destination;
+
+  int source_bpp = 0;
+  int dest_bpp = 0;
+
+  switch (babl_source->instance.class_type)
+    {
+      case BABL_FORMAT:
+        source_bpp = babl_source->format.bytes_per_pixel;
+        break;
+      case BABL_TYPE:
+        source_bpp = babl_source->type.bits / 8;
+        break;
+      default:
+        babl_log ("=eeek{%i}\n", babl_source->instance.class_type - BABL_MAGIC);
+    }
+
+  switch (babl_destination->instance.class_type)
+    {
+      case BABL_FORMAT:
+        dest_bpp = babl_destination->format.bytes_per_pixel;
+        break;
+      case BABL_TYPE:
+        dest_bpp = babl_destination->type.bits / 8;
+        break;
+      default:
+        babl_log ("-eeek{%i}\n", babl_destination->instance.class_type - BABL_MAGIC);
+     }
+
   if (!fpi->init_instrumentation_done)
     {
       /* this initialization can be done only once since the
        * source and destination formats do not change during
        * the search */
-      Babl *fmt_source = (Babl *) BABL (babl_list_get_first (path))->conversion.source;
-      Babl *fmt_destination = (Babl *) BABL (babl_list_get_last (path))->conversion.destination;
-      init_path_instrumentation (fpi, fmt_source, fmt_destination);
+      init_path_instrumentation (fpi, babl_source, babl_destination);
       fpi->init_instrumentation_done = 1;
     }
 
   /* calculate this path's view of what the result should be */
   ticks_start = babl_ticks ();
-  process_conversion_path (path, fpi->source, fpi->destination, NUM_TEST_PIXELS);
+  process_conversion_path (path, fpi->source, source_bpp, fpi->destination, dest_bpp, fpi->num_test_pixels);
   ticks_end = babl_ticks ();
   *path_cost = babl_process_cost (ticks_start, ticks_end);
 
@@ -605,20 +673,20 @@ get_path_instrumentation (FishPathInstrumentation *fpi,
    * for comparison with each other
    */
   babl_process (fpi->fish_destination_to_rgba,
-                fpi->destination, fpi->destination_rgba_double, NUM_TEST_PIXELS);
+                fpi->destination, fpi->destination_rgba_double, fpi->num_test_pixels);
 
   *path_error = babl_rel_avg_error (fpi->destination_rgba_double,
                                     fpi->ref_destination_rgba_double,
-                                    NUM_TEST_PIXELS * 4);
+                                    fpi->num_test_pixels * 4);
 
 #if 0
   fpi->fish_rgba_to_source->fish.processings--;
   fpi->fish_reference->fish.processings--;
   fpi->fish_destination_to_rgba->fish.processings -= 2;
 
-  fpi->fish_rgba_to_source->fish.pixels      -= NUM_TEST_PIXELS;
-  fpi->fish_reference->fish.pixels           -= NUM_TEST_PIXELS;
-  fpi->fish_destination_to_rgba->fish.pixels -= 2 * NUM_TEST_PIXELS;
+  fpi->fish_rgba_to_source->fish.pixels      -= fpi->num_test_pixels;
+  fpi->fish_reference->fish.pixels           -= fpi->num_test_pixels;
+  fpi->fish_destination_to_rgba->fish.pixels -= 2 * fpi->num_test_pixels;
 #endif
 
   *ref_cost = fpi->reference_cost;
