@@ -252,7 +252,7 @@ static double legal_error (void)
 
   env = getenv ("BABL_TOLERANCE");
   if (env && env[0] != '\0')
-    error = atof (env);
+    error = babl_parse_double (env);
   else
     error = BABL_LEGAL_ERROR;
   return error;
@@ -308,7 +308,7 @@ babl_fish_stats (FILE *file)
            "<html>\n"
            "<head>\n"
            "<title>BablFishPath introspection</title>\n"
-
+           "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n"
            "<style type='text/css'>"
            " body {"
            "   font-family: sans;"
