@@ -40,6 +40,7 @@
  */
 
 #include <stdlib.h>
+#include <stdint.h>
 
 #include "config.h"
 #include "babl.h"
@@ -54,8 +55,8 @@
 
 static float         table_8_F[1 << 8];
 static float         table_8g_F[1 << 8];
-static unsigned char table_F_8[1 << 16];
-static unsigned char table_F_8g[1 << 16];
+static unsigned char table_F_8[1 << 17];
+static unsigned char table_F_8g[1 << 17];
 
 static int           table_inited = 0;
 
@@ -80,51 +81,55 @@ table_init (void)
   {
     union
     {
-      float          f;
-      unsigned short s[2];
+      float    f;
+      uint32_t s;
     } u;
     u.f = 0.0;
 
-    u.s[0] = 0.0;
+    //u.s[0] = 0;
 
-    for (u.s[1] = 0; u.s[1] < 65535; u.s[1] += 1)
+    for (u.s = 0; u.s < 4294900000U; u.s += 32768)
       {
-        unsigned char c;
-        unsigned char cg;
+        int c;
+        int cg;
 
         if (u.f <= 0.0)
           {
             c  = 0;
             cg = 0;
           }
-        else if (u.f >= 1.0)
-          {
-            c  = 255;
-            cg = 255;
-          }
         else
           {
-            c  = rint (u.f * 255.0);
-            cg = rint (linear_to_gamma_2_2 (u.f) * 255.0);
+            c  = (u.f * 255.1619) + 0.5;
+            cg = (linear_to_gamma_2_2 (u.f) * 255.1619) + 0.5;
+            if (cg > 255) cg = 255;
+            if (c > 255) c = 255;
           }
 
-        table_F_8[u.s[1]]  = c;
-        table_F_8g[u.s[1]] = cg;
+        table_F_8[(u.s >> 15) & ((1 << 17)-1)]  = c;
+        table_F_8g[(u.s >> 15) & ((1 << 17)-1)] = cg;
       }
   }
+#if 0
   /* fix tables to ensure 1:1 conversions back and forth */
   if (0)
     {
       int i;
       for (i = 0; i < 256; i++)
         {
-          float           f  = table_8_F[i];
+          float           f;
           unsigned short *hi = ((unsigned short *) (void *) &f);
           unsigned short *lo = ((unsigned short *) (void *) &f);
+
+          f = table_8_F[i];
           *lo              = 0;
-          table_F_8[(*hi)] = i;
+          table_F_8[*hi] = i;
+          f  = table_8g_F[i];
+          *lo              = 0;
+          table_F_8g[*hi] = i;
         }
     }
+#endif
 }
 
 /* function to find the index in table for a float */
@@ -134,10 +139,10 @@ gggl_float_to_index16 (float f)
   union
   {
     float          f;
-    unsigned short s[2];
+    uint32_t       s;
   } u;
   u.f = f;
-  return u.s[1];
+  return (u.s >> 15) & ((1 << 17)-1);
 }
 
 static INLINE long
@@ -323,67 +328,6 @@ conv_rgb8_rgbaF (unsigned char *src, unsigned char *dst, long samples)
   return samples;
 }
 
-static long
-conv_rgbAF_sdl32 (unsigned char *srcc,
-                  unsigned char *dstc,
-                  long           samples)
-{
-  float         *src = (void *) srcc;
-  unsigned char *dst = (void *) dstc;
-  long           n   = samples;
-
-  while (n--)
-    {
-      int   i;
-      float alpha = src[3];
-      for (i = 0; i < 3; i++)
-        {
-          float ca = src[i];
-          int   ret;
-          if (alpha < BABL_ALPHA_THRESHOLD)
-            ret = 0;
-          else
-            ret = table_F_8g[gggl_float_to_index16 (ca / alpha)];
-          if (ret <= 0)
-            dst[i] = 0;
-          else if (ret > 255)
-            dst[i] = 255;
-          else
-            dst[i] = ret;
-        }
-      {
-        int t = dst[0];
-        dst[0] = dst[2];
-        dst[2] = t;
-        dst[3] = 255;
-      }
-      src += 4;
-      dst += 4;
-    }
-  return samples;
-}
-
-
-static long
-conv_rgbaF_sdl32 (unsigned char *srcc,
-                  unsigned char *dstc,
-                  long           samples)
-{
-  float         *src = (void *) srcc;
-  unsigned char *dst = (void *) dstc;
-  long           n   = samples;
-
-  while (n--)
-    {
-      dst[0] = table_F_8g[gggl_float_to_index16 (src[2])];
-      dst[1] = table_F_8g[gggl_float_to_index16 (src[1])];
-      dst[2] = table_F_8g[gggl_float_to_index16 (src[0])];
-      src   += 4;
-      dst   += 4;
-    }
-  return samples;
-}
-
 static long
 conv_rgbAF_rgb8 (unsigned char *srcc,
                  unsigned char *dstc,
@@ -473,9 +417,11 @@ conv_rgbAF_rgbaF (unsigned char *srcc,
   while (n--)
     {
       float alpha = src[3];
-      float recip = 1.0/alpha;
+      float recip;
       if (alpha < BABL_ALPHA_THRESHOLD)
         recip = 0.0;
+      else
+        recip = 1.0/alpha;
       dst[0] = src[0] * recip;
       dst[1] = src[1] * recip;
       dst[2] = src[2] * recip;
@@ -518,9 +464,9 @@ conv_rgbAF_lrgba8 (unsigned char *srcc,
   return samples;
 }
 
-
-
 #define conv_rgb8_rgbAF    conv_rgb8_rgbaF
+#define conv_gamma_rgbaF_gamma_rgbAF   conv_rgbaF_rgbAF
+#define conv_gamma_rgbAF_gamma_rgbaF   conv_rgbAF_rgbaF
 
 int init (void);
 
@@ -543,6 +489,24 @@ init (void)
     babl_component ("Ba"),
     babl_component ("A"),
     NULL);
+    
+    
+  const Babl *gamma_rgbaF = babl_format_new (
+    babl_model ("R'G'B'A"),
+    babl_type ("float"),
+    babl_component ("R'"),
+    babl_component ("G'"),
+    babl_component ("B'"),
+    babl_component ("A"),
+    NULL);
+  const Babl *gamma_rgbAF = babl_format_new (
+    babl_model ("R'aG'aB'aA"),
+    babl_type ("float"),
+    babl_component ("R'a"),
+    babl_component ("G'a"),
+    babl_component ("B'a"),
+    babl_component ("A"),
+    NULL);
 
   const Babl *lrgba8 = babl_format_new (
     babl_model ("RGBA"),
@@ -577,15 +541,6 @@ init (void)
     babl_component ("G'"),
     babl_component ("B'"),
     NULL);
-  const Babl *sdl32 = babl_format_new (
-    "name", "B'aG'aR'aPAD u8",
-    babl_model ("R'G'B'"),
-    babl_type ("u8"),
-    babl_component ("B'"),
-    babl_component ("G'"),
-    babl_component ("R'"),
-    babl_component ("PAD"),
-    NULL);
 
   table_init ();
 
@@ -594,14 +549,16 @@ init (void)
 
   o (rgbaF, rgbAF);
   o (rgbAF, rgbaF);
+  
+  o (gamma_rgbaF, gamma_rgbAF);
+  o (gamma_rgbAF, gamma_rgbaF);
+  
   o (rgbAF, lrgba8);
   o (rgb8, rgbaF);
   o (rgb8, rgbAF);
   o (rgba8, rgbaF);
-  o (rgbaF, sdl32);
   o (rgbaF, rgb8);
   o (rgbAF, rgb8);
-  o (rgbAF, sdl32);
   o (bgrA8, rgba8);
 
   return 0;
